@@ -46,6 +46,7 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE MultiWayIf #-}
 {-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE CPP #-}
 #if __GLASGOW_HASKELL__ >= 710
@@ -59,8 +60,34 @@
 #endif
 
 module Data.GenValidity
-    ( module Data.Validity
-    , module Data.GenValidity
+    ( GenUnchecked(..)
+    , GenValid(..)
+    , GenInvalid(..)
+
+    -- * Helper functions
+    , genValidStructurally
+    , genValidStructurallyWithoutExtraChecking
+    , shrinkValidStructurally
+    , shrinkValidStructurallyWithoutExtraFiltering
+    , module Data.GenValidity.Utils
+
+    -- * Re-exports
+    , module Data.Validity
+
+    -- * The Generics magic
+    , genericGenUnchecked
+    , GGenUnchecked(..)
+    , genericShrinkUnchecked
+    , uncheckedRecursivelyShrink
+    , GUncheckedRecursivelyShrink(..)
+    , uncheckedSubterms
+    , GUncheckedSubterms(..)
+    , GUncheckedSubtermsIncl(..)
+    , GGenValid(..)
+    , GValidRecursivelyShrink(..)
+    , structurallyValidSubterms
+    , GValidSubterms(..)
+    , GValidSubtermsIncl(..)
     ) where
 
 import Data.Validity
@@ -68,6 +95,7 @@ import Data.Validity
 import Data.Fixed (Fixed(..), HasResolution)
 #if MIN_VERSION_base(4,9,0)
 import Data.List.NonEmpty (NonEmpty((:|)))
+import qualified Data.List.NonEmpty as NE
 #endif
 #if MIN_VERSION_base(4,8,0)
 import Data.Word (Word8, Word16, Word32, Word64)
@@ -83,17 +111,21 @@ import Test.QuickCheck hiding (Fixed)
 
 #if MIN_VERSION_base(4,8,0)
 import GHC.Natural
-import Control.Monad (forM)
 #else
 import Control.Applicative ((<*>), (<$>), pure)
-import Control.Monad (forM)
 #endif
 
+import Data.GenValidity.Utils
+
 {-# ANN module "HLint: ignore Reduce duplication" #-}
 
 -- | A class of types for which truly arbitrary values can be generated.
 --
--- === Automatic instances with 'Generic'
+-- === How to instantiate 'GenUnchecked'
+--
+-- __Step 1__: Try to instantiate 'GenUnchecked' via 'Generic'.
+--         __this is probably what you want__
+--
 -- An instance of this class can be made automatically if the type in question
 -- has a 'Generic' instance. This instance will try to use 'genUnchecked' to
 -- generate all structural sub-parts of the value that is being generated.
@@ -102,7 +134,7 @@ import Control.Monad (forM)
 --
 -- > {-# LANGUAGE DeriveGeneric #-}
 -- >
--- > data MyType = MyType Double String
+-- > data MyType = MyType Rational String
 -- >     deriving (Show, Eq, Generic)
 -- >
 -- > instance GenUnchecked MyType
@@ -111,25 +143,61 @@ import Control.Monad (forM)
 --
 -- > instance GenUnchecked MyType where
 -- >     genUnchecked = MyType <$> genUnchecked <*> genUnchecked
+--
+--
+-- __Step 2__: If an instatiation via 'Generic' is not possible, then you should emulate what
+--         'genericGenUnchecked' does.
+--         This means that all sub-parts should be  generated using 'genUnchecked'.
+--         Make sure to generate any possible value, valid or not, that can exist at runtime
+--         even when taking the existence of 'Unsafe.Coerce.unsafeCoerce' into account.
+--
+-- === Warning: Invalid values can be funky
+--
+-- Some types have serious validity constraints. See 'Text' or 'ByteString' for example.
+-- These can behave very strangely when they are not valid.
+-- In that case, __do not override 'GenUnchecked' such that 'genUnchecked' only generates valid values__.
+-- In that case, do not override 'genUnchecked' at all.
+-- Instead, use 'genValid' from 'GenValid' (see below) instead.
 class GenUnchecked a where
     genUnchecked :: Gen a
     default genUnchecked :: (Generic a, GGenUnchecked (Rep a)) =>
         Gen a
-    genUnchecked = to <$> gGenUnchecked
+    genUnchecked = genericGenUnchecked
 
     shrinkUnchecked :: a -> [a]
     default shrinkUnchecked ::
         (Generic a, GUncheckedRecursivelyShrink (Rep a), GUncheckedSubterms (Rep a) a) =>
         a -> [a]
-    shrinkUnchecked = gShrinkUnchecked
+    shrinkUnchecked = genericShrinkUnchecked
 
 -- | A class of types for which valid values can be generated.
 --
+-- === How to instantiate 'GenValid'
+--
+-- __Step 1__: Try to instantiate 'GenValid' without overriding any functions.
+--             It is possible that, if few values are valid or if validity
+--             checking is expensive, that the resulting generator is too slow.
+--             In that case, go to Step 2.
+--
+-- __Step 2__: Try to instantiate 'GenValid' using the helper functions via 'Generic'
+--             This involves using 'genValidStructurally' to override 'genValid' and
+--             using 'shrinkValidStructurally' to override 'shrinkValid'.
+--             __Every time you override 'genValid', you should also override 'shrinkValid'__
+--
+-- __Step 3__: If the above is not possible due to lack of a 'Generic' instance,
+--             then you should emulate what 'genValidStructurally' does.
+--             This means that all sub-parts should be generated using 'genValid'.
+--             Make sure to generate any possible valid value, but only valid values.
+--
+-- === A note about 'Arbitrary'
+--
 -- If you also write @Arbitrary@ instances for @GenValid@ types, it may be
--- best to simply write @arbitrary = genValid@.
+-- best to simply use
+--
+-- > arbitrary = genValid
+-- > shrink = shrinkValid
 class (Validity a, GenUnchecked a) =>
       GenValid a where
-    genValid :: Gen a
     -- | Generate a valid datum, this should cover all possible valid values in
     -- the type
     --
@@ -140,12 +208,35 @@ class (Validity a, GenUnchecked a) =>
     -- To speed up testing, it may be a good idea to implement this yourself.
     -- If you do, make sure that it is possible to generate all possible valid
     -- data, otherwise your testing may not cover all cases.
+    genValid :: Gen a
     genValid = genUnchecked `suchThat` isValid
 
+    -- | Shrink a valid value.
+    --
+    -- The default implementation is as follows:
+    --
+    -- >  shrinkValid = filter isValid . shrinkUnchecked
+    --
+    -- It is important that this shrinking function only shrinks values to valid values.
+    -- If `shrinkValid` ever shrinks a value to an invalid value, the test that is being shrunk for
+    -- might fail for a different reason than for the reason that it originally failed.
+    -- This would lead to very confusing error messages.
     shrinkValid :: a -> [a]
     shrinkValid = filter isValid . shrinkUnchecked
 
 -- | A class of types for which invalid values can be generated.
+--
+-- === How to instantiate 'GenInvalid'
+--
+-- __Step 1__: Realise that you probably do not want to.
+--             It makes no sense, and serves no purpose, to instantiate 'GenInvalid' for types
+--             which contain no invalid values. (In fact, the default implementation will go into
+--             an infinite loop for such types.)
+--             You should only instantiate 'GenInvalid' if you explicitly want to use it
+--             to write tests that deal with invalid values, or if you are writing a container
+--             for parametric values.
+--
+-- __Step 2__: Instantiate 'GenInvalid' without overriding any functions.
 class (Validity a, GenUnchecked a) =>
       GenInvalid a where
     genInvalid :: Gen a
@@ -169,6 +260,9 @@ instance (GenUnchecked a, GenUnchecked b) => GenUnchecked (a, b) where
             a <- resize r genUnchecked
             b <- resize s genUnchecked
             return (a, b)
+    shrinkUnchecked (a, b) = ((,) <$> shrinkUnchecked a <*> shrinkUnchecked b)
+      ++ [ (a', b) | a' <- shrinkUnchecked a ]
+      ++ [ (a, b') | b' <- shrinkUnchecked b ]
 
 instance (GenValid a, GenValid b) => GenValid (a, b) where
     genValid =
@@ -177,6 +271,9 @@ instance (GenValid a, GenValid b) => GenValid (a, b) where
             a <- resize r genValid
             b <- resize s genValid
             return (a, b)
+    shrinkValid (a, b) = ((,) <$> shrinkValid a <*> shrinkValid b)
+      ++ [ (a', b) | a' <- shrinkValid a ]
+      ++ [ (a, b') | b' <- shrinkValid b ]
 
 instance (GenInvalid a, GenInvalid b) => GenInvalid (a, b) where
     genInvalid =
@@ -193,9 +290,13 @@ instance (GenInvalid a, GenInvalid b) => GenInvalid (a, b) where
 
 instance (GenUnchecked a, GenUnchecked b) => GenUnchecked (Either a b) where
     genUnchecked = oneof [Left <$> genUnchecked, Right <$> genUnchecked]
+    shrinkUnchecked (Left a) = Left <$> shrinkUnchecked a
+    shrinkUnchecked (Right b) = Right <$> shrinkUnchecked b
 
 instance (GenValid a, GenValid b) => GenValid (Either a b) where
     genValid = oneof [Left <$> genValid, Right <$> genValid]
+    shrinkValid (Left a) = Left <$> shrinkValid a
+    shrinkValid (Right b) = Right <$> shrinkValid b
 
 -- | This instance ensures that the generated tupse contains at least one invalid element. The other element is unchecked.
 instance (GenInvalid a, GenInvalid b) => GenInvalid (Either a b) where
@@ -210,6 +311,10 @@ instance (GenUnchecked a, GenUnchecked b, GenUnchecked c) =>
             b <- resize s genUnchecked
             c <- resize t genUnchecked
             return (a, b, c)
+    shrinkUnchecked (a, b, c) =
+        [ (a', b', c')
+        | (a', (b', c')) <- shrinkUnchecked (a, (b, c))
+        ]
 
 instance (GenValid a, GenValid b, GenValid c) => GenValid (a, b, c) where
     genValid =
@@ -219,6 +324,10 @@ instance (GenValid a, GenValid b, GenValid c) => GenValid (a, b, c) where
             b <- resize s genValid
             c <- resize t genValid
             return (a, b, c)
+    shrinkValid (a, b, c) =
+        [ (a', b', c')
+        | (a', (b', c')) <- shrinkValid (a, (b, c))
+        ]
 
 -- | This instance ensures that the generated triple contains at least one invalid element. The other two are unchecked.
 instance (GenInvalid a, GenInvalid b, GenInvalid c) =>
@@ -251,6 +360,10 @@ instance (GenUnchecked a, GenUnchecked b, GenUnchecked c, GenUnchecked d) =>
             c <- resize t genUnchecked
             d <- resize u genUnchecked
             return (a, b, c, d)
+    shrinkUnchecked (a, b, c, d) =
+        [ (a', b', c', d')
+        | (a', (b', (c', d'))) <- shrinkUnchecked (a, (b, (c, d)))
+        ]
 
 instance (GenValid a, GenValid b, GenValid c, GenValid d) =>
          GenValid (a, b, c, d) where
@@ -262,6 +375,10 @@ instance (GenValid a, GenValid b, GenValid c, GenValid d) =>
             c <- resize t genValid
             d <- resize u genValid
             return (a, b, c, d)
+    shrinkValid (a, b, c, d) =
+        [ (a', b', c', d')
+        | (a', (b', (c', d'))) <- shrinkValid (a, (b, (c, d)))
+        ]
 
 -- | This instance ensures that the generated triple contains at least one invalid element. The other two are unchecked.
 instance (GenInvalid a, GenInvalid b, GenInvalid c, GenInvalid d) =>
@@ -303,6 +420,10 @@ instance (GenUnchecked a, GenUnchecked b, GenUnchecked c, GenUnchecked d, GenUnc
             d <- resize u genUnchecked
             e <- resize v genUnchecked
             return (a, b, c, d, e)
+    shrinkUnchecked (a, b, c, d, e) =
+        [ (a', b', c', d', e')
+        | (a', (b', (c', (d', e')))) <- shrinkUnchecked (a, (b, (c, (d, e))))
+        ]
 
 instance (GenValid a, GenValid b, GenValid c, GenValid d, GenValid e) =>
          GenValid (a, b, c, d, e) where
@@ -315,6 +436,10 @@ instance (GenValid a, GenValid b, GenValid c, GenValid d, GenValid e) =>
             d <- resize u genValid
             e <- resize v genValid
             return (a, b, c, d, e)
+    shrinkValid (a, b, c, d, e) =
+        [ (a', b', c', d', e')
+        | (a', (b', (c', (d', e')))) <- shrinkValid (a, (b, (c, (d, e))))
+        ]
 
 -- | This instance ensures that the generated triple contains at least one invalid element. The other two are unchecked.
 instance (GenInvalid a, GenInvalid b, GenInvalid c, GenInvalid d, GenInvalid e) =>
@@ -357,38 +482,52 @@ instance (GenInvalid a, GenInvalid b, GenInvalid c, GenInvalid d, GenInvalid e)
 
 instance GenUnchecked a => GenUnchecked (Maybe a) where
     genUnchecked = oneof [pure Nothing, Just <$> genUnchecked]
+    shrinkUnchecked Nothing = []
+    shrinkUnchecked (Just a) = Nothing : (Just <$> shrinkUnchecked a)
+
 
 instance GenValid a => GenValid (Maybe a) where
     genValid = oneof [pure Nothing, Just <$> genValid]
+    shrinkValid Nothing = []
+    shrinkValid (Just a) = Nothing : (Just <$> shrinkValid a)
 
 instance GenInvalid a => GenInvalid (Maybe a) where
     genInvalid = Just <$> genInvalid
 
+#if MIN_VERSION_base(4,9,0)
+instance GenUnchecked a => GenUnchecked (NonEmpty a) where
+    genUnchecked = do
+      l <- genUnchecked
+      case NE.nonEmpty l of
+        Nothing -> scale (+1) genUnchecked
+        Just ne -> pure ne
+    shrinkUnchecked (v :| vs) = [ e :| es | (e, es) <- shrinkUnchecked (v, vs)]
+
+instance GenValid a => GenValid (NonEmpty a) where
+    genValid = do
+      l <- genValid
+      case NE.nonEmpty l of
+        Nothing -> scale (+1) genValid
+        Just ne -> pure ne
+    shrinkValid (v :| vs) = [ e :| es | (e, es) <- shrinkValid (v, vs)]
+
+instance GenInvalid a => GenInvalid (NonEmpty a) where
+    genInvalid = do
+      l <- genInvalid
+      case NE.nonEmpty l of
+        Nothing -> scale (+1) genInvalid
+        Just ne -> pure ne
+#endif
+
 instance GenUnchecked a => GenUnchecked [a] where
     genUnchecked = genListOf genUnchecked
     shrinkUnchecked = shrinkList shrinkUnchecked
 
-#if MIN_VERSION_base(4,9,0)
-instance GenUnchecked a => GenUnchecked (NonEmpty a) where
-    genUnchecked = (:|) <$> genUnchecked <*> genUnchecked
-    shrinkUnchecked (v :| vs) = [ e :| es | (e, es) <- shrinkUnchecked (v, vs)]
-
-instance GenValid a => GenValid (NonEmpty a) where
-    genValid = (:|) <$> genValid <*> genValid
-
-instance GenInvalid a => GenInvalid (NonEmpty a) where
-    genInvalid = sized $ \n -> do
-      (a, b) <- genSplit n
-      oneof
-        [ (:|) <$> resize a genUnchecked <*> resize b genInvalid
-        , (:|) <$> resize a genInvalid <*> resize b genUnchecked
-        ]
-#endif
-
 -- | If we can generate values of a certain type, we can also generate lists of
 -- them.
 instance GenValid a => GenValid [a] where
     genValid = genListOf genValid
+    shrinkValid = shrinkList shrinkValid
 
 -- | This instance ensures that the generated list contains at least one element
 -- that satisfies 'isInvalid'. The rest is unchecked.
@@ -487,33 +626,34 @@ instance GenUnchecked Word64 where
 instance GenValid Word64
 
 instance GenUnchecked Float where
-    genUnchecked = arbitrary
+    genUnchecked = frequency [(9, arbitrary), (1, elements [read "NaN", read "Infinity", read "-Infinity", read "-0"])]
 #if MIN_VERSION_QuickCheck(2,9,2)
-    shrinkUnchecked = shrink
+    shrinkUnchecked f = if
+      | isInfinite f -> []
+      | isNaN f -> []
+      | otherwise -> shrink f
 #else
     shrinkUnchecked _ = []
 #endif
 
 instance GenValid Float where
-    genValid = arbitrary
-
--- | Either 'NaN' or 'Infinity'.
-instance GenInvalid Float where
-    genInvalid = elements [read "NaN", read "Infinity", read "-Infinity", read "-0"]
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Double where
-    genUnchecked = arbitrary
+    genUnchecked = frequency [(9, arbitrary), (1, elements [read "NaN", read "Infinity", read "-Infinity", read "-0"])]
 #if MIN_VERSION_QuickCheck(2,9,2)
-    shrinkUnchecked = shrink
+    shrinkUnchecked d = if
+      | isInfinite d -> []
+      | isNaN d -> []
+      | otherwise -> shrink d
 #else
     shrinkUnchecked _ = []
 #endif
 
-instance GenValid Double
-
--- | Either 'NaN' or 'Infinity'.
-instance GenInvalid Double where
-    genInvalid = elements [read "NaN", read "Infinity", read "-Infinity", read "-0"]
+instance GenValid Double where
+    genValid = genUnchecked
+    shrinkValid = shrinkUnchecked
 
 instance GenUnchecked Integer where
     genUnchecked = arbitrary
@@ -532,17 +672,11 @@ instance GenValid Natural where
 #endif
 
 instance (Integral a, GenUnchecked a) => GenUnchecked (Ratio a) where
-    genUnchecked = do
-        n <- genUnchecked
-        d <- genUnchecked
-        pure $ n :% d
+    genUnchecked = (:%) <$> genUnchecked <*> genUnchecked
     shrinkUnchecked (n :% d) = [n' :% d' | (n', d') <- shrinkUnchecked (n, d)]
 
 instance (Integral a, Num a, Ord a, GenValid a) => GenValid (Ratio a) where
-    genValid = do
-      n <- genValid
-      d <- genValid `suchThat` (> 0)
-      pure $ n % d
+    genValid = (%) <$> genValid <*> (genValid `suchThat` (> 0))
     shrinkValid (n :% d) = [n' % d' | (n', d') <- shrinkValid (n, d), d' > 0]
 
 instance (Integral a, Num a, Ord a, GenValid a) => GenInvalid (Ratio a)
@@ -553,77 +687,8 @@ instance HasResolution a => GenUnchecked (Fixed a) where
 
 instance HasResolution a => GenValid (Fixed a)
 
-shrinkT2
-  :: (a -> [a])
-  -> (a, a) -> [(a, a)]
-shrinkT2 s (a, b) = (,) <$> s a <*> s b
-
-shrinkT3
-  :: (a -> [a])
-  -> (a, a, a) -> [(a, a, a)]
-shrinkT3 s (a, b, c) = (,,) <$> s a <*> s b <*> s c
-
-
--- | 'upTo' generates an integer between 0 (inclusive) and 'n'.
-upTo :: Int -> Gen Int
-upTo n
-    | n <= 0 = pure 0
-    | otherwise = choose (0, n)
-
--- | 'genSplit a' generates a tuple '(b, c)' such that 'b + c' equals 'a'.
-genSplit :: Int -> Gen (Int, Int)
-genSplit n
-    | n < 0 = pure (0, 0)
-    | otherwise = do
-        i <- choose (0, n)
-        let j = n - i
-        pure (i, j)
-
--- | 'genSplit3 a' generates a triple '(b, c, d)' such that 'b + c + d' equals 'a'.
-genSplit3 :: Int -> Gen (Int, Int, Int)
-genSplit3 n
-    | n < 0 = pure (0, 0, 0)
-    | otherwise = do
-        (a, z) <- genSplit n
-        (b, c) <- genSplit z
-        return (a, b, c)
-
--- | 'genSplit4 a' generates a quadruple '(b, c, d, e)' such that 'b + c + d + e' equals 'a'.
-genSplit4 :: Int -> Gen (Int, Int, Int, Int)
-genSplit4 n
-    | n < 0 = pure (0, 0, 0, 0)
-    | otherwise = do
-        (y, z) <- genSplit n
-        (a, b) <- genSplit y
-        (c, d) <- genSplit z
-        return (a, b, c, d)
-
--- | 'genSplit5 a' generates a quadruple '(b, c, d, e, f)' such that 'b + c + d + e + f' equals 'a'.
-genSplit5 :: Int -> Gen (Int, Int, Int, Int, Int)
-genSplit5 n
-    | n < 0 = pure (0, 0, 0, 0, 0)
-    | otherwise = do
-        (y, z) <- genSplit n
-        (a, b, c) <- genSplit3 y
-        (d, e) <- genSplit z
-        return (a, b, c, d, e)
-
--- | 'arbPartition n' generates a list 'ls' such that 'sum ls' equals 'n'.
-arbPartition :: Int -> Gen [Int]
-arbPartition k
-    | k <= 0 = pure []
-    | otherwise = do
-        first <- choose (1, k)
-        rest <- arbPartition $ k - first
-        return $ first : rest
-
--- | A version of @listOf@ that takes size into account more accurately.
-genListOf :: Gen a -> Gen [a]
-genListOf func =
-    sized $ \n -> do
-        size <- upTo n
-        pars <- arbPartition size
-        forM pars $ \i -> resize i func
+genericGenUnchecked :: (Generic a, GGenUnchecked (Rep a)) => Gen a
+genericGenUnchecked = to <$> gGenUnchecked
 
 class GGenUnchecked f where
     gGenUnchecked :: Gen (f a)
@@ -632,10 +697,7 @@ instance GGenUnchecked U1 where
     gGenUnchecked = pure U1
 
 instance (GGenUnchecked a, GGenUnchecked b) => GGenUnchecked (a :*: b) where
-    gGenUnchecked = do
-        g1 <- gGenUnchecked
-        g2 <- gGenUnchecked
-        pure $ g1 :*: g2
+    gGenUnchecked = (:*:) <$> gGenUnchecked <*> gGenUnchecked
 
 instance (GGenUnchecked a, GGenUnchecked b) => GGenUnchecked (a :+: b) where
     gGenUnchecked = oneof [L1 <$> gGenUnchecked, R1 <$> gGenUnchecked]
@@ -649,8 +711,8 @@ instance (GenUnchecked a) => GGenUnchecked (K1 i a) where
 
 -- | Shrink a term to any of its immediate subterms,
 -- and also recursively shrink all subterms.
-gShrinkUnchecked :: (Generic a, GUncheckedRecursivelyShrink (Rep a), GUncheckedSubterms (Rep a) a) => a -> [a]
-gShrinkUnchecked x = uncheckedSubterms x ++ uncheckedRecursivelyShrink x
+genericShrinkUnchecked :: (Generic a, GUncheckedRecursivelyShrink (Rep a), GUncheckedSubterms (Rep a) a) => a -> [a]
+genericShrinkUnchecked x = uncheckedSubterms x ++ uncheckedRecursivelyShrink x
 
 -- | Recursively shrink all immediate uncheckedSubterms.
 uncheckedRecursivelyShrink :: (Generic a, GUncheckedRecursivelyShrink (Rep a)) => a -> [a]
@@ -661,7 +723,9 @@ class GUncheckedRecursivelyShrink f where
 
 instance (GUncheckedRecursivelyShrink f, GUncheckedRecursivelyShrink g) => GUncheckedRecursivelyShrink (f :*: g) where
   gUncheckedRecursivelyShrink (x :*: y) =
-      [x' :*: y' | x' <- gUncheckedRecursivelyShrink x, y' <- gUncheckedRecursivelyShrink y]
+    ((:*:) <$> gUncheckedRecursivelyShrink x <*> gUncheckedRecursivelyShrink y)
+      ++ [ x' :*: y | x' <- gUncheckedRecursivelyShrink x ]
+      ++ [ x :*: y' | y' <- gUncheckedRecursivelyShrink y ]
 
 instance (GUncheckedRecursivelyShrink f, GUncheckedRecursivelyShrink g) => GUncheckedRecursivelyShrink (f :+: g) where
   gUncheckedRecursivelyShrink (L1 x) = map L1 (gUncheckedRecursivelyShrink x)
@@ -735,3 +799,148 @@ instance OVERLAPPING_ GUncheckedSubtermsIncl (K1 i a) a where
 instance OVERLAPPING_ GUncheckedSubtermsIncl (K1 i a) b where
   gUncheckedSubtermsIncl (K1 _) = []
 
+
+-- | Generate a valid value by generating all the sub parts using the 'Generic' instance,
+-- and trying that until a valid value has been generated
+--
+-- > genValidStructurally = genValidStructurallyWithoutExtraChecking `suchThat` isValid
+--
+-- This is probably the function that you are looking for.
+-- If you do use this function to override `genValid`, you probably also want to use
+-- 'shrinkValidStructurally' to override 'shrinkValid'.
+genValidStructurally :: (Validity a, Generic a, GGenValid (Rep a)) => Gen a
+genValidStructurally = genValidStructurallyWithoutExtraChecking `suchThat` isValid
+
+-- | Generate a valid value by generating all the sub parts using the 'Generic' instance,
+--
+-- This generator is _not_ guaranteed to generate a valid value.
+--
+-- This is probably _not_ the function that you are looking for when overriding
+-- `genValid` _unless_ the type in question has no _extra_ validity constraints on top of
+-- the validity of its sub parts.
+genValidStructurallyWithoutExtraChecking :: (Generic a, GGenValid (Rep a)) => Gen a
+genValidStructurallyWithoutExtraChecking = to <$> gGenValid
+
+class GGenValid f where
+    gGenValid :: Gen (f a)
+
+instance GGenValid U1 where
+    gGenValid = pure U1
+
+instance (GGenValid a, GGenValid b) => GGenValid (a :*: b) where
+    gGenValid = (:*:) <$> gGenValid <*> gGenValid
+
+instance (GGenValid a, GGenValid b) => GGenValid (a :+: b) where
+    gGenValid = oneof [L1 <$> gGenValid, R1 <$> gGenValid]
+
+instance (GGenValid a) => GGenValid (M1 i c a) where
+    gGenValid = M1 <$> gGenValid
+
+instance (GenValid a) => GGenValid (K1 i a) where
+    gGenValid = K1 <$> genValid
+
+
+-- | Shrink a term to any of its immediate valid subterms,
+-- and also recursively shrink all subterms, and then filtering out the results that are not valid.
+--
+-- > shrinkValidStructurally = filter isValid . shrinkValidStructurallyWithoutExtraFiltering
+--
+-- This is probably the function that you are looking for.
+shrinkValidStructurally :: (Validity a, Generic a, GValidRecursivelyShrink (Rep a), GValidSubterms (Rep a) a) => a -> [a]
+shrinkValidStructurally = filter isValid . shrinkValidStructurallyWithoutExtraFiltering
+
+-- | Shrink a term to any of its immediate valid subterms,
+-- and also recursively shrink all subterms.
+--
+-- This shrinking function is _not_ guaranteed to shrink to valid values.
+--
+-- This is probably _not_ the function that you are looking for when overriding
+-- `shrinkValid` _unless_ the type in question has no _extra_ validity constraints on top of
+-- the validity of its sub parts.
+shrinkValidStructurallyWithoutExtraFiltering :: (Generic a, GValidRecursivelyShrink (Rep a), GValidSubterms (Rep a) a) => a -> [a]
+shrinkValidStructurallyWithoutExtraFiltering x = structurallyValidSubterms x ++ structurallyValidRecursivelyShrink x
+
+-- | Recursively shrink all immediate structurally valid subterms.
+structurallyValidRecursivelyShrink :: (Generic a, GValidRecursivelyShrink (Rep a)) => a -> [a]
+structurallyValidRecursivelyShrink = map to . gValidRecursivelyShrink . from
+
+class GValidRecursivelyShrink f where
+  gValidRecursivelyShrink :: f a -> [f a]
+
+instance (GValidRecursivelyShrink f, GValidRecursivelyShrink g) => GValidRecursivelyShrink (f :*: g) where
+  gValidRecursivelyShrink (x :*: y) =
+    ((:*:) <$> gValidRecursivelyShrink x <*> gValidRecursivelyShrink y)
+      ++ [ x' :*: y | x' <- gValidRecursivelyShrink x ]
+      ++ [ x :*: y' | y' <- gValidRecursivelyShrink y ]
+
+instance (GValidRecursivelyShrink f, GValidRecursivelyShrink g) => GValidRecursivelyShrink (f :+: g) where
+  gValidRecursivelyShrink (L1 x) = map L1 (gValidRecursivelyShrink x)
+  gValidRecursivelyShrink (R1 x) = map R1 (gValidRecursivelyShrink x)
+
+instance GValidRecursivelyShrink f => GValidRecursivelyShrink (M1 i c f) where
+  gValidRecursivelyShrink (M1 x) = map M1 (gValidRecursivelyShrink x)
+
+instance GenValid a => GValidRecursivelyShrink (K1 i a) where
+  gValidRecursivelyShrink (K1 x) = map K1 (shrinkValid x)
+
+instance GValidRecursivelyShrink U1 where
+  gValidRecursivelyShrink U1 = []
+
+instance GValidRecursivelyShrink V1 where
+  -- The empty type can't be shrunk to anything.
+  gValidRecursivelyShrink _ = []
+
+
+-- | All immediate validSubterms of a term.
+structurallyValidSubterms :: (Generic a, GValidSubterms (Rep a) a) => a -> [a]
+structurallyValidSubterms = gValidSubterms . from
+
+
+class GValidSubterms f a where
+  gValidSubterms :: f a -> [a]
+
+instance GValidSubterms V1 a where
+  gValidSubterms _ = []
+
+instance GValidSubterms U1 a where
+  gValidSubterms U1 = []
+
+instance (GValidSubtermsIncl f a, GValidSubtermsIncl g a) => GValidSubterms (f :*: g) a where
+  gValidSubterms (l :*: r) = gValidSubtermsIncl l ++ gValidSubtermsIncl r
+
+instance (GValidSubtermsIncl f a, GValidSubtermsIncl g a) => GValidSubterms (f :+: g) a where
+  gValidSubterms (L1 x) = gValidSubtermsIncl x
+  gValidSubterms (R1 x) = gValidSubtermsIncl x
+
+instance GValidSubterms f a => GValidSubterms (M1 i c f) a where
+  gValidSubterms (M1 x) = gValidSubterms x
+
+instance GValidSubterms (K1 i a) b where
+  gValidSubterms (K1 _) = []
+
+
+class GValidSubtermsIncl f a where
+  gValidSubtermsIncl :: f a -> [a]
+
+instance GValidSubtermsIncl V1 a where
+  gValidSubtermsIncl _ = []
+
+instance GValidSubtermsIncl U1 a where
+  gValidSubtermsIncl U1 = []
+
+instance (GValidSubtermsIncl f a, GValidSubtermsIncl g a) => GValidSubtermsIncl (f :*: g) a where
+  gValidSubtermsIncl (l :*: r) = gValidSubtermsIncl l ++ gValidSubtermsIncl r
+
+instance (GValidSubtermsIncl f a, GValidSubtermsIncl g a) => GValidSubtermsIncl (f :+: g) a where
+  gValidSubtermsIncl (L1 x) = gValidSubtermsIncl x
+  gValidSubtermsIncl (R1 x) = gValidSubtermsIncl x
+
+instance GValidSubtermsIncl f a => GValidSubtermsIncl (M1 i c f) a where
+  gValidSubtermsIncl (M1 x) = gValidSubtermsIncl x
+
+-- This is the important case: We've found a term of the same type.
+instance OVERLAPPING_ GValidSubtermsIncl (K1 i a) a where
+  gValidSubtermsIncl (K1 x) = [x]
+
+instance OVERLAPPING_ GValidSubtermsIncl (K1 i a) b where
+  gValidSubtermsIncl (K1 _) = []
